@@ -19,6 +19,7 @@
  * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  */
 
+import Adw from 'gi://Adw';
 import GObject from 'gi://GObject';
 import Gdk from 'gi://Gdk';
 import GeocodeGlib from 'gi://GeocodeGlib';
@@ -37,6 +38,7 @@ import * as GeocodeFactory from './geocode.js';
 import {GeoJSONShapeLayer} from './geoJSONShapeLayer.js';
 import {KmlShapeLayer} from './kmlShapeLayer.js';
 import {GpxShapeLayer} from './gpxShapeLayer.js';
+import {FitShapeLayer} from './fitShapeLayer.js';
 import {Location} from './location.js';
 import * as MapSource from './mapSource.js';
 import {MapWalker} from './mapWalker.js';
@@ -55,7 +57,6 @@ import {TransitWalkMarker} from './transitWalkMarker.js';
 import {TurnPointMarker} from './turnPointMarker.js';
 import {UserLocationMarker} from './userLocationMarker.js';
 import * as Utils from './utils.js';
-import {ZoomInDialog} from './zoomInDialog.js';
 
 const _LOCATION_STORE_TIMEOUT = 500;
 const MapMinZoom = 2;
@@ -87,7 +88,8 @@ export class MapView extends Gtk.Overlay {
     static MapType = {
         LOCAL: 'MapsLocalSource',
         STREET: 'MapsStreetSource',
-        AERIAL: 'MapsAerialSource'
+        AERIAL: 'MapsAerialSource',
+        VECTOR: 'MapsVectorSource',
     }
 
     /*
@@ -134,6 +136,7 @@ export class MapView extends Gtk.Overlay {
 
         this._mainWindow = mainWindow;
         this._storeId = 0;
+        this._storeRotationId = 0;
         this.map = this._initMap();
 
         this.child = this.map;
@@ -144,8 +147,10 @@ export class MapView extends Gtk.Overlay {
         this._initScale();
         this._initLayers();
 
-        if (Application.normalStartup)
+        if (Application.normalStartup) {
             this._goToStoredLocation();
+            this._setStoredRotation();
+        }
 
         this.shapeLayerStore = new Gio.ListStore(GObject.TYPE_OBJECT);
 
@@ -191,12 +196,12 @@ export class MapView extends Gtk.Overlay {
 
         this._clickGesture = new Gtk.GestureClick({ button: Gdk.BUTTON_SECONDARY });
         this._clickGesture.connect('pressed', this._onClickGesturePressed.bind(this));
-        this.add_controller(this._clickGesture);
+        this.map.add_controller(this._clickGesture);
 
         this._longPressGesture = new Gtk.GestureLongPress({ touch_only: true });;
         this._longPressGesture.connect('pressed',
                                        this._onLongPressGesturePressed.bind(this));
-        this.add_controller(this._longPressGesture);
+        this.map.add_controller(this._longPressGesture);
     }
 
     vfunc_size_allocate(width, height, baseline) {
@@ -267,6 +272,7 @@ export class MapView extends Gtk.Overlay {
         map.viewport.min_zoom_level = MapMinZoom;
 
         map.viewport.connect('notify::latitude', this._onViewMoved.bind(this));
+        map.viewport.connect('notify::rotation', this._onViewRotated.bind(this));
         // switching map type will set view min-zoom-level from map source
         map.viewport.connect('notify::min-zoom-level', () => {
             if (map.viewport.min_zoom_level < MapMinZoom) {
@@ -342,6 +348,7 @@ export class MapView extends Gtk.Overlay {
         ShapeLayer.SUPPORTED_TYPES.push(GeoJSONShapeLayer);
         ShapeLayer.SUPPORTED_TYPES.push(KmlShapeLayer);
         ShapeLayer.SUPPORTED_TYPES.push(GpxShapeLayer);
+        ShapeLayer.SUPPORTED_TYPES.push(FitShapeLayer);
 
         this._routeLayers = [];
     }
@@ -407,7 +414,7 @@ export class MapView extends Gtk.Overlay {
             }
         }
 
-        return MapType.STREET;
+        return MapView.MapType.STREET;
     }
 
     getMapType() {
@@ -418,8 +425,6 @@ export class MapView extends Gtk.Overlay {
         if (this._mapType && this._mapType === mapType)
             return;
 
-        //let overlay_sources = this.view.get_overlay_sources();
-
         this._mapType = mapType;
 
         let mapSource;
@@ -429,15 +434,11 @@ export class MapView extends Gtk.Overlay {
 
             if (mapType === MapView.MapType.AERIAL && tiles.aerial)
                 mapSource = MapSource.createAerialSource();
+            else if (mapType === MapView.MapType.VECTOR && Shumate.VectorRenderer.is_supported())
+                mapSource = MapSource.createVectorSource();
             else
                 mapSource = MapSource.createStreetSource();
 
-            // update license
-            if (this._mapSource)
-                this._license.remove_map_source(this._mapSource);
-
-            this._license.append_map_source(mapSource);
-
             Application.settings.set('map-type', mapType);
         } else {
             let source = new GnomeMaps.FileDataSource({
@@ -457,17 +458,30 @@ export class MapView extends Gtk.Overlay {
             } catch(e) {
                 this.setMapType(MapView.MapType.STREET);
                 Application.application.local_tile_path = false;
-                Utils.showDialog(e.message, Gtk.MessageType.ERROR, this._mainWindow);
+                this._mainWindow.showToast(e.message);
                 return;
             }
         }
 
-        let mapLayer = new Shumate.MapLayer({ map_source: mapSource,
-                                                  viewport:   this.map.viewport });
+        const mapLayer = new Shumate.MapLayer({ map_source: mapSource,
+                                                viewport:   this.map.viewport });
 
-        this.map.add_layer(mapLayer);
-        this._mapSource = mapSource;
+        if (this._mapLayer) {
+            this.map.insert_layer_above(mapLayer, this._mapLayer);
+            this.map.remove_layer(this._mapLayer);
+        } else {
+            this.map.add_layer(mapLayer);
+        }
+
+        this._mapLayer = mapLayer;
+
+        if (this._mapSource)
+            this._license.remove_map_source(this._mapSource);
+        this._license.append_map_source(mapSource);
         this.map.viewport.set_reference_map_source(mapSource);
+
+        this._mapSource = mapSource;
+
         this.emit("map-type-changed", mapType);
     }
 
@@ -498,8 +512,7 @@ export class MapView extends Gtk.Overlay {
 
     _onShapeLoad(error, bbox, layer) {
         if (error) {
-            let msg = _("Failed to open layer");
-            Utils.showDialog(msg, Gtk.MessageType.ERROR, this._mainWindow);
+            this._mainWindow.showToast(_("Failed to open layer"));
         } else {
             bbox.compose(layer.bbox);
         }
@@ -515,19 +528,25 @@ export class MapView extends Gtk.Overlay {
         if (result.confirmLoad) {
             let totalFileSizeMB = result.totalFileSizeMB;
 
-            let dialog = new Gtk.MessageDialog ({
+            let dialog = new Adw.MessageDialog ({
                 transient_for: this._mainWindow,
                 modal: true,
-                buttons: Gtk.ButtonsType.OK_CANCEL,
-                text: _("Do you want to continue?"), 
-                secondary_text: _("You are about to open files with a total " + 
-                                 "size of %s MB. This could take some time to" +
-                                 " load").format(totalFileSizeMB.toLocaleString(undefined,
-                                                                               { maximumFractionDigits: 1 }))
+                heading: _("Do you want to continue?"),
+                body: _("You are about to open files with a total " +
+                        "size of %s MB. This could take some time to" +
+                        " load").format(totalFileSizeMB.toLocaleString(undefined,
+                                                                       { maximumFractionDigits: 1 }))
             });
 
+            dialog.add_response('cancel', _("Cancel"));
+            dialog.add_response('continue', _("Continue"));
+            dialog.set_response_appearance('continue',
+                                           Adw.ResponseAppearance.SUGGESTED);
+            dialog.set_default_response('cancel');
+            dialog.set_close_response('cancel');
+
             dialog.connect('response', (widget, responseId) => {
-                if (responseId === Gtk.ResponseType.OK) {
+                if (responseId === 'continue') {
                     this._loadShapeLayers(files);
                 }
                 dialog.destroy();
@@ -558,8 +577,7 @@ export class MapView extends Gtk.Overlay {
                 }
             } catch (e) {
                 Utils.debug(e);
-                let msg = _("Failed to open layer");
-                Utils.showDialog(msg, Gtk.MessageType.ERROR, this._mainWindow);
+                this._mainWindow.showToast(_("Failed to open layer"));
             }
         }
     }
@@ -590,8 +608,7 @@ export class MapView extends Gtk.Overlay {
             this._placeLayer.add_marker(marker);
             marker.goToAndSelect(true);
         } catch(e) {
-            let msg = _("Failed to open GeoURI");
-            Utils.showDialog(msg, Gtk.MessageType.ERROR, this._mainWindow);
+            this._mainWindow.showToast(_("Failed to open GeoURI"));
             Utils.debug("failed to open GeoURI: %s".format(e.message));
         }
     }
@@ -605,7 +622,7 @@ export class MapView extends Gtk.Overlay {
                 this._placeLayer.add_marker(marker);
                 marker.goToAndSelect(true);
             } else {
-                Utils.showDialog(error, Gtk.MessageType.ERROR, this._mainWindow);
+                this._mainWindow.showToast(error);
             }
         });
     }
@@ -690,6 +707,14 @@ export class MapView extends Gtk.Overlay {
         }
     }
 
+    _storeRotation() {
+        let viewport = this.map.viewport;
+        let rotation = viewport.rotation;
+
+        if (!isNaN(rotation))
+            Application.settings.set('rotation', rotation);
+    }
+
     _goToStoredLocation() {
         let location = Application.settings.get('last-viewed-location');
 
@@ -723,6 +748,18 @@ export class MapView extends Gtk.Overlay {
         }
     }
 
+    _setStoredRotation() {
+        let rotation = Application.settings.get('rotation');
+
+        if (rotation < 0.0 || rotation >= 2 * Math.PI) {
+            // safeguard agains out-of-bounds rotation values
+            Utils.debug('Invalid stored rotation, set no rotation');
+            rotation = 0;
+        }
+
+        this.map.viewport.rotation = rotation;
+    }
+
     gotoBBox(bbox, linear) {
         if (!bbox.isValid()) {
             Utils.debug('Bounding box is invalid');
@@ -954,6 +991,16 @@ export class MapView extends Gtk.Overlay {
         });
     }
 
+    _onViewRotated() {
+        if (this._storeRotationId !== 0)
+            return;
+
+        this._storeRotationId = GLib.timeout_add(null, _LOCATION_STORE_TIMEOUT, () => {
+            this._storeRotationId = 0;
+            this._storeRotation();
+        });
+    }
+
     onSetMarkerSelected(selectedMarker) {
         this.emit('marker-selected', selectedMarker);
     }
@@ -1030,9 +1077,7 @@ export class MapView extends Gtk.Overlay {
             if (place) {
                 this.showPlace(place, true);
             } else {
-                let msg = _("Nothing found here!");
-
-                Utils.showDialog(msg, Gtk.MessageType.INFO, this._mainWindow);
+                this._mainWindow.showToast(_("Nothing found here!"));
             }
         });
     }
@@ -1071,15 +1116,17 @@ export class MapView extends Gtk.Overlay {
         let viewport = this.map.viewport;
 
         if (viewport.zoom_level < OSMEdit.MIN_ADD_LOCATION_ZOOM_LEVEL) {
-            let zoomInDialog =
-                new ZoomInDialog({ longitude: this._longitude,
-                                   latitude: this._latitude,
-                                   map: this.map,
-                                   transient_for: this._mainWindow,
-                                   modal: true });
+            let zoomInToast =
+                new Adw.Toast({ title: _("Zoom in to add location"),
+                                button_label: _("Zoom In") });
+
+            zoomInToast.connect('button-clicked', () => {
+                this.map.go_to_full(this._latitude, this._longitude,
+                                    OSMEdit.MIN_ADD_LOCATION_ZOOM_LEVEL);
+            });
+
+            this._mainWindow.addToast(zoomInToast);
 
-            zoomInDialog.connect('response', () => zoomInDialog.destroy());
-            zoomInDialog.show();
             return;
         }
 
@@ -1091,8 +1138,7 @@ export class MapView extends Gtk.Overlay {
         dialog.connect('response', (dialog, response) => {
             dialog.destroy();
             if (response === OSMEditDialog.Response.UPLOADED) {
-                Utils.showDialog(_("Location was added to the map, note that it may take a while before it shows on the map and in search results."),
-                                 Gtk.MessageType.INFO, this._mainWindow);
+                this._mainWindow.showToast(_("Location was added in OpenStreetMap"));
             }
         });
     }
